/* 
 * Eneso Human Interface Device Library
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib.h>

#ifdef G_OS_WIN32
#include "ehid_windows.h"
#define READ_BUFFER_OFFSET	1
#else
#define READ_BUFFER_OFFSET	0
#include "ehid_linux.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include "ehid_private.h"

ConnectionStatus	status	= UNKNOWN;

EHidConnectedCb		user_connected_cb		= NULL;
EHidDisconnectedCb	user_disconnected_cb	= NULL;
EHidMessageCb		user_message_cb			= NULL;

gchar*	user_name			= NULL;
guint	user_vid 			= 0;
guint 	user_pid 			= 0;
guint 	user_input_length	= 0;
guint 	user_output_length	= 0;

gchar*	read_buffer = NULL;


	void 
ehid_init (gchar* name, guint vid, guint pid, guint input_length, guint output_length,
		EHidConnectedCb connected_cb, EHidDisconnectedCb disconnected_cb, EHidMessageCb message_cb)
{
	/* Allocate memory for the read buffer */
	read_buffer = malloc (input_length);

	/* Set the callbacks */
	user_connected_cb		= connected_cb;
	user_disconnected_cb	= disconnected_cb;
	user_message_cb			= message_cb;

	user_name			= g_strdup (name);
	user_vid			= vid;
	user_pid			= pid;
	user_input_length	= input_length;
	user_output_length	= output_length;

	/* Initialize */
#ifdef G_OS_WIN32
	ehid_windows_init ();
#else
	ehid_linux_init ();
#endif

	/* Look for the device and call the appropriate callback */
	check_connected_and_notify ();
}

	void 
ehid_terminate (void)
{
#ifdef G_OS_WIN32
	ehid_windows_terminate ();
#else
	ehid_linux_terminate ();
#endif 

	g_free (user_name);
	g_free (read_buffer);
}

	void 
ehid_search (void)
{
#ifdef G_OS_WIN32
#else
	ehid_linux_search ();
#endif 
}

	void
ehid_receive (void)
{
	if (status != CONNECTED)
		return;

#ifdef G_OS_WIN32
	ehid_windows_receive ();
#else
	ehid_linux_receive ();
#endif
}

	void 
ehid_send (gchar* message)
{
	if (status != CONNECTED)
		return;

#ifdef G_OS_WIN32
	ehid_windows_send (message);
#else
	ehid_linux_send (message);
#endif
}

	void
check_connected_and_notify (void)
{
	ConnectionStatus	status_before;
	guint	vid_found;
	guint	pid_found;

	status_before = status;

#ifdef G_OS_WIN32	
	status = ehid_windows_check (&vid_found, &pid_found);
#else
	vid_found = user_vid;
	pid_found = user_pid;

	status = ehid_linux_check ();
#endif

	/* Notify the application about the device connection or disconnection,
	 * but do not notify the same event more than once */
	if (status != status_before)
	{
		if (status == CONNECTED && user_connected_cb)
			user_connected_cb (vid_found, pid_found);
		else if (status == DISCONNECTED && user_disconnected_cb)
			user_disconnected_cb ();
	}
}
