/* 
 * Eneso Single Program Instance Library
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <dbus/dbus.h>
#include <dbus/dbus-glib-lowlevel.h>
#include "esingle.h"

#define STR_DUPLICATE_OBJECT_PATH	"/esingle/duplicate"
#define STR_INTERFACE_DUPLICATE		"esingle.interface.duplicate"
#define STR_SIGNAL_NAME				"esingle_instance"

static DBusConnection* dbus_connection = NULL;

static ESingleInstanceCb	user_instance_cb	= NULL;

static DBusHandlerResult
dbus_signal_cb	(DBusConnection* connection, DBusMessage* message, void* user_data);



	int
esingle_init (const char* name, ESingleInstanceCb instance_cb)
{
	DBusError	err = { 0 };
	int			ret;

	user_instance_cb = instance_cb;

#define CHECK_DBUS_ERROR \
	if (dbus_error_is_set (&err)) \
	{ g_message (err.message); dbus_error_free (&err); return TRUE; } \

	dbus_connection = dbus_bus_get (DBUS_BUS_SESSION, &err);
	CHECK_DBUS_ERROR;

	dbus_connection_setup_with_g_main (dbus_connection, NULL);

	ret = dbus_bus_request_name (dbus_connection, name, DBUS_NAME_FLAG_DO_NOT_QUEUE, &err);
	CHECK_DBUS_ERROR;

	if (ret == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
	{
		dbus_bus_add_match (dbus_connection,
				"type='signal',interface='" STR_INTERFACE_DUPLICATE "',path='" STR_DUPLICATE_OBJECT_PATH "'", &err);
		CHECK_DBUS_ERROR;

		dbus_connection_add_filter (dbus_connection, dbus_signal_cb, NULL, NULL);

		return TRUE;
	}
	else
	{
		DBusMessage* message = NULL;

		message = dbus_message_new_signal (STR_DUPLICATE_OBJECT_PATH, STR_INTERFACE_DUPLICATE, STR_SIGNAL_NAME);
		dbus_connection_send (dbus_connection, message, NULL);

		dbus_message_unref (message);

		return FALSE;
	}

#undef CHECK_DBUS_ERROR
}

	void
esingle_terminate (void)
{
}

	static DBusHandlerResult
dbus_signal_cb (DBusConnection* connection, DBusMessage* message, void* user_data)
{
	if (dbus_message_is_signal (message, STR_INTERFACE_DUPLICATE, STR_SIGNAL_NAME))
	{
		if (user_instance_cb)
			user_instance_cb ();

		return DBUS_HANDLER_RESULT_HANDLED;
	}

	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}
