/* 
 * enCore 
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <cairo.h>
#include <gtk/gtk.h>
#include <math.h>
#include "draw.h"

static PangoLayout*
create_text_layout (cairo_t* cr, const gchar* text, const gchar* font);


	void 
draw_text (cairo_t* cr, gint x, gint y, const gchar* text, const gchar* font, TextAlign align)
{
	PangoLayout* 	layout;
	gint			width, height;

	layout = create_text_layout (cr, text, font);

	/* Move to the position */
	pango_layout_get_size (layout, &width, &height);

	switch (align)
	{
		case TEXT_ALIGN_LEFT:
			cairo_move_to (cr, x, y - ((float) height / PANGO_SCALE) / 2);
			break;
		case TEXT_ALIGN_RIGHT:
			cairo_move_to (cr, x - ((float) width / PANGO_SCALE),
					y - ((float) height / PANGO_SCALE) / 2);
			break;
		case TEXT_ALIGN_CENTER:
			cairo_move_to (cr, x - ((float) width / PANGO_SCALE) / 2,
					y - ((float) height / PANGO_SCALE) / 2);
			break;
	}

	/* Draw */
	pango_cairo_show_layout (cr, layout);

	g_object_unref (layout);
}

	void 
draw_text_adjust (cairo_t* cr, gint x, gint y, gint max_width, const gchar* text, const gchar* font)
{
	PangoLayout* 	layout;
	gint			width, height;
	gint			width_scale, height_scale;

	layout = create_text_layout (cr, text, font);

	pango_layout_get_size (layout, &width, &height);
	width_scale = (gint) ((gfloat) width / PANGO_SCALE);
	height_scale = (gint) ((gfloat) height / PANGO_SCALE);

	if (width_scale < max_width)
	{
		/* Center */
		cairo_move_to (cr, x + ((max_width - width_scale) >> 1), y - (height_scale >> 1));
	}
	else
	{
		/* Clip */
		cairo_rectangle (cr, x, y - (height_scale >> 1), max_width, height_scale);
		cairo_clip (cr);

		/* Align left */
		cairo_move_to (cr, x, y - (height_scale >> 1));
	}
		
	pango_cairo_show_layout (cr, layout);

	g_object_unref (layout);
}

	void
draw_text_multiline (cairo_t* cr, gint x, gint y, const gchar* text,
		const gchar* font, gint text_width, gboolean center_y)
{
	PangoLayout*	layout;
	gint			width, height;

	layout = create_text_layout (cr, text, font);

	/* Configure the multiline layout */
	pango_layout_set_width (layout, (float) text_width * PANGO_SCALE);
	pango_layout_set_spacing (layout, 3.0 * PANGO_SCALE);

	/* Move to the position */
	pango_layout_get_size (layout, &width, &height);

	if (center_y)
		cairo_move_to (cr, x, y - ((float) height / PANGO_SCALE) / 2);
	else
		cairo_move_to (cr, x, y);

	/* Draw */
	pango_cairo_show_layout (cr, layout);

	g_object_unref (layout);
}

	void
draw_round_rect (cairo_t* cr, gint x, gint y, gint width, gint height, gint radius)
{
	cairo_move_to (cr, x, y + height - radius);
	cairo_arc (cr, x + radius, y + radius, radius, M_PI, 1.5 * M_PI);
	cairo_arc (cr, x + width - radius, y + radius, radius, 1.5 * M_PI, 2.0 * M_PI);
	cairo_arc (cr, x + width - radius, y + height - radius, radius, 0, 0.5 * M_PI);
	cairo_arc (cr, x + radius, y + height - radius, radius, 0.5 * M_PI, M_PI);
}

	static PangoLayout*
create_text_layout (cairo_t* cr, const gchar* text, const gchar* font)
{
	PangoLayout*			layout;
	PangoFontDescription*	desc;
	PangoAttrList*			attrs;
	gchar*					txt;

	layout = pango_cairo_create_layout (cr);

	/* Parse the text */
	attrs = pango_layout_get_attributes (layout);
	pango_parse_markup (text, -1, 0, &attrs, &txt, NULL, NULL);
	pango_layout_set_attributes (layout, attrs);
	pango_attr_list_unref (attrs);

	/* Use the text without markups */
	pango_layout_set_text (layout, txt, -1);

	/* Load the font */
	desc = pango_font_description_from_string (font);
	pango_layout_set_font_description (layout, desc);
	pango_font_description_free (desc);

	pango_cairo_update_layout (cr, layout);

	return layout;
}
