#include <glib.h>
#include <stdio.h>
#include <string.h>
#include "misc.h"
#include "layout.h"

gint	current_keyb_layout;
gchar*	keyb_layout_name [N_KEYB_LAYOUTS] =
{ "en_US", "en_UK", "el", "es", "he", "de", "it", "pt" };
gchar*	keyb_layout_print_name [N_KEYB_LAYOUTS] =
{ "US", "UK", "EL", "ES", "HE", "DE", "IT", "PT" };
GSequence*	keyb_layout [N_KEYB_LAYOUTS];

static gint
load_layout	(GSequence** layout, gchar* file);
static void
free_key	(gpointer data);


	void
load_keyb_layouts (void)
{
	gchar*	data_file;
	gchar*	file;
	gint	i;
	
	for (i = 0; i < N_KEYB_LAYOUTS; i++)
	{
		file = g_strdup_printf ("/layouts/%s", keyb_layout_name [i]);
		data_file = get_data_file (file);
		g_free (file);

		load_layout (&(keyb_layout [i]), data_file);

		g_free (data_file);
	}
}

	static gint
load_layout (GSequence** layout, gchar* file)
{
	FILE*			f;
	gchar			line [100];
	gchar*			token;
	guchar			code;
	gchar*			text;
	Key*			key;
	GSequenceIter*	iter;

	/* Create the sequence */
	*layout = g_sequence_new (free_key);

	/* Open the file */
	f = (FILE*) g_fopen (file, "r");
	if (!f)
		return -1;

	/* Read the keys */
	while (!feof (f))
	{
		/* Read a line */
		if (fgets (line, sizeof (line), f) == NULL)
			continue;

		/* Read the code */
		token = strtok (line, " \t");
		sscanf (token, "%x", &code);

		/* Read the key */
		if (!token)
			continue;

		/* Get the text */
		text = NULL;
		token = strtok (NULL, "[");

		if (token)
		{
			text = token;

			token++;
			while (*token && *token != ']')
				token++;

			*token = 0;
			token++;
		}

		if (text != NULL && *text != 0)
		{
			key = g_malloc (sizeof (Key));

			key->text = g_strdup (text);
			key->code = code;

			g_sequence_append (*layout, key);
		}
	}

	fclose (f);

	return 0;
}

	static void
free_key (gpointer data)
{
	Key*	key;

	key = (Key*) data;

	g_free (key->text);
	g_free (key);
}
