/* 
 * enCore
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <libintl.h>
#include <locale.h>
#include <stdlib.h>
#include "ehid.h"
#include "esingle.h"
#include "configuration.h"
#include "encore.h"
#include "help.h"
#include "misc.h"
#include "protocol.h"
#include "window.h"

static void
terminate	(void);

static gboolean
hid_idle_search_cb	(gpointer data);
static gboolean
hid_idle_receive_cb	(gpointer data);

static void
ehid_connected_cb		(guint vid, guint pid);
static void
ehid_disconnected_cb	(void);
static void
ehid_message_cb			(gchar* message, guint n_bytes);

static void
esingle_instance_cb		(void);

	
	int
main (int argc, char* argv [])
{
	gchar*	data_file;

	/* Check if there is another instance running */
	if (!esingle_init ("Eneso.enCore.Single", esingle_instance_cb))
		return 0;
	
	/* GTK+ */
	gtk_init (&argc, &argv);

	/* Find the data and config directories */
	find_data_dir ();
	find_config_dir ();

	/* Window icon */
	data_file = get_data_file ("/icons/encore.ico");
	gtk_window_set_default_icon_from_file (data_file, NULL);
	g_free (data_file);

	/* Load the configuration */
	load_config_file ();
	load_keyb_layouts ();

	/* Create the windows */
	load_main_window ();
	create_help ();

	/* HID communications */
	ehid_init ("encore", 0xEE50, 0x0102, 9, 9,
			ehid_connected_cb, ehid_disconnected_cb, ehid_message_cb);

	/* Show main window */
	show_main_window ();
	update_main_window ();

	/* Main loop */
	g_atexit (terminate);
	gtk_main ();

	return 0;
}

	static void
terminate (void)
{
	ehid_terminate ();
	esingle_terminate ();
}

	static gboolean
hid_idle_search_cb	(gpointer data)
{
	ehid_search ();

	return !device_connected;
}

	static gboolean
hid_idle_receive_cb	(gpointer data)
{
	ehid_receive ();

	return device_connected;
}

	static void 
ehid_connected_cb (guint vid, guint pid)
{
	device_connected = TRUE;
	update_main_window ();

	g_idle_add_full (G_PRIORITY_LOW, hid_idle_receive_cb, NULL, NULL);
}

	static void
ehid_disconnected_cb (void)
{
	device_connected = FALSE;
	update_main_window ();

#ifdef G_OS_UNIX
	g_idle_add_full (G_PRIORITY_LOW, hid_idle_search_cb, NULL, NULL);
#endif
}

	static void
ehid_message_cb (gchar* message, guint n_bytes)
{
	handle_message (message, n_bytes);
}

	static void
esingle_instance_cb (void)
{
	show_main_window ();
}
