/* 
 * enCore
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib.h>
#include <glib/gi18n.h>
#include <cairo.h>
#include <gtk/gtk.h>
#include <string.h>
#include "config.h"
#include "encore.h"
#include "configuration.h"
#include "layout.h"
#include "misc.h"
#include "protocol.h"
#include "window.h"


typedef struct
{
	gchar*	text;
	Action	action;
}
ComboAction;

#define N_MOUSE_COMBO_ACTIONS	8

static const ComboAction mouse_combo_actions [N_MOUSE_COMBO_ACTIONS] =
{
	{ "Left click",		{ MOUSE_ACTION,		MOUSE_LEFT_CLICK,		0 } },
	{ "Right click",	{ MOUSE_ACTION,		MOUSE_RIGHT_CLICK,		0 } },
	{ "Double click",	{ MOUSE_ACTION,		MOUSE_DOUBLE_CLICK,		0 } },
	{ "Drag/Drop",		{ MOUSE_ACTION,		MOUSE_DRAG_DROP,		0 } },
	{ "Move up",		{ MOUSE_MOV_ACTION,	MOV_UP,					0 } },
	{ "Move down",		{ MOUSE_MOV_ACTION,	MOV_DOWN,				0 } },
	{ "Move right",		{ MOUSE_MOV_ACTION,	MOV_RIGHT,				0 } },
	{ "Move left",		{ MOUSE_MOV_ACTION,	MOV_LEFT,				0 } }	
};

#define N_JOY_COMBO_ACTIONS	12

static const ComboAction joy_combo_actions [N_JOY_COMBO_ACTIONS] =
{
	{ "Button 1",		{ JOYSTICK_ACTION,		0,			0 } },
	{ "Button 2",		{ JOYSTICK_ACTION,		1,			0 } },
	{ "Button 3",		{ JOYSTICK_ACTION,		2,			0 } },
	{ "Button 4",		{ JOYSTICK_ACTION,		3,			0 } },
	{ "Button 5",		{ JOYSTICK_ACTION,		4,			0 } },
	{ "Button 6",		{ JOYSTICK_ACTION,		5,			0 } },
	{ "Button 7",		{ JOYSTICK_ACTION,		6,			0 } },
	{ "Button 8",		{ JOYSTICK_ACTION,		7,			0 } },
	{ "Move up",		{ JOYSTICK_MOV_ACTION,	MOV_UP,		0 } },
	{ "Move down",		{ JOYSTICK_MOV_ACTION,	MOV_DOWN,	0 } },
	{ "Move left",		{ JOYSTICK_MOV_ACTION,	MOV_LEFT,	0 } },
	{ "Move right",		{ JOYSTICK_MOV_ACTION,	MOV_RIGHT,	0 } }
};

#define N_GEN_COMBO_ACTIONS	8

static const ComboAction gen_combo_actions [N_GEN_COMBO_ACTIONS] =
{
	{ "Event 1",		{ GENERIC_ACTION,		1,			0 } },
	{ "Event 2",		{ GENERIC_ACTION,		2,			0 } },
	{ "Event 3",		{ GENERIC_ACTION,		3,			0 } },
	{ "Event 4",		{ GENERIC_ACTION,		4,			0 } },
	{ "Event 5",		{ GENERIC_ACTION,		5,			0 } },
	{ "Event 6",		{ GENERIC_ACTION,		6,			0 } },
	{ "Event 7",		{ GENERIC_ACTION,		7,			0 } },
	{ "Event 8",		{ GENERIC_ACTION,		8,			0 } },
};


typedef struct
{
	gchar*	text;
	gint	speed [3];
	gint	stretch [2];
}
MouseComboSpeed;

#define N_MOUSE_COMBO_SPEEDS	4

static const MouseComboSpeed mouse_combo_speeds [N_MOUSE_COMBO_SPEEDS] =
{
	{ "Constant (slow)",		{ 5, 5, 5 },	{ 40, 150 } },
	{ "Constant (fast)",		{ 12, 12, 12 },	{ 40, 150 } },
	{ "Accelerate",				{ 7, 18, 18 },	{ 40, 150 } },
	{ "Accelerate and brake",	{ 5, 18, 7 },	{ 40, 150 } }
};


typedef struct
{
	gchar*	text;
	gint	value;
}
JoyComboDisplacement;

#define N_JOY_COMBO_DISPLACEMENTS 5

static const JoyComboDisplacement joy_combo_displacements [N_JOY_COMBO_DISPLACEMENTS] = 
{
	{ "25%",	32 },
	{ "50%",	64 },
	{ "75%",	93 },
	{ "90%",	115 },
	{ "100%",	127 }
};

#define PAGE_MOUSE	0
#define PAGE_GEN	1
#define PAGE_KEYB	2
#define PAGE_JOY	3

static struct
{
	GtkWidget* 	window_main;
	GtkWidget*	checkmenuitem_dev_options;
	GtkWidget*	radiomenuitem_lang [N_LANGUAGES];
	GtkWidget*	radiomenuitem_keyb_layout [N_KEYB_LAYOUTS];
	GtkWidget* 	image_symbol [N_SWITCHES];
	GtkWidget* 	radiobutton_mouse [N_SWITCHES];
	GtkWidget* 	radiobutton_keyb [N_SWITCHES];
	GtkWidget* 	radiobutton_joy [N_SWITCHES];
	GtkWidget* 	radiobutton_gen [N_SWITCHES];
	GtkWidget* 	notebook [N_SWITCHES];
	GtkWidget* 	alignment_mouse [N_SWITCHES];
	GtkWidget* 	alignment_keyb [N_SWITCHES];
	GtkWidget* 	alignment_joy [N_SWITCHES];
	GtkWidget* 	alignment_gen [N_SWITCHES];
	GtkWidget* 	checkbutton_control [N_SWITCHES];
	GtkWidget* 	checkbutton_shift [N_SWITCHES];
	GtkWidget* 	checkbutton_alt [N_SWITCHES];
	GtkWidget* 	combobox_mouse [N_SWITCHES];
	GtkWidget* 	combobox_keyb [N_SWITCHES];
	GtkWidget* 	combobox_joy [N_SWITCHES];
	GtkWidget* 	combobox_gen [N_SWITCHES];
	GtkWidget* 	checkbutton_auto_release [N_SWITCHES];
	GtkWidget*	label_mouse_speed;
	GtkWidget*	alignment_mouse_speed;
	GtkWidget*	combobox_mouse_speed;
	GtkWidget*	label_joy_displacement;
	GtkWidget*	alignment_joy_displacement;
	GtkWidget*	combobox_joy_displacement;
	GtkWidget*	button_read;
	GtkWidget*	button_send;

	GtkWidget* 	window_about;
	GtkWidget* 	label_version;
	GtkWidget* 	image_encore;
	GtkWidget* 	image_eneso;
}
widgets;

static void
combobox_action_changed_cb (GtkComboBox* widget, gpointer user_data);
static void
set_checkbutton_auto_release_visible (gint sw);



	void
load_main_window (void)
{
	GtkBuilder*	builder;
	GtkWidget*	widget;
	gchar*		data_file;
	gchar		text [100];
	gint		i, j;
	GSequenceIter*	iter;
	Key*		key;

	/* Load the interface files */
	builder = gtk_builder_new ();

	data_file = get_data_file ("interface/main.glade");
	gtk_builder_add_from_file (builder, data_file, NULL);
	g_free (data_file);

	data_file = get_data_file ("interface/about.glade");
	gtk_builder_add_from_file (builder, data_file, NULL);
	g_free (data_file);

	/* Get the widgets */
#define GET_WIDGET(widget,name) \
	widgets.widget = GTK_WIDGET (gtk_builder_get_object (builder, name)); \
	if (widgets.widget == NULL) g_warning ("Widget not found: %s", name);

	GET_WIDGET (window_main, "window_main");
	GET_WIDGET (checkmenuitem_dev_options, "checkmenuitem_dev_options");

	for (i = 0; i < N_LANGUAGES; i++)
	{
		sprintf (text, "radiomenuitem_lang_%s", language_name [i]);
		GET_WIDGET (radiomenuitem_lang [i], text);

		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (widgets.radiomenuitem_lang [i]),
				i == current_language);
	}

	for (i = 0; i < N_KEYB_LAYOUTS; i++)
	{
		sprintf (text, "radiomenuitem_keyb_layout_%s", keyb_layout_name [i]);
		GET_WIDGET (radiomenuitem_keyb_layout [i], text);

		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (widgets.radiomenuitem_keyb_layout [i]),
				i == current_keyb_layout);
	}

	for (i = 0; i < N_SWITCHES; i++)
	{
		sprintf (text, "image_symbol_%d", i + 1);
		GET_WIDGET (image_symbol [i], text);

		sprintf (text, "radiobutton_mouse_%d", i + 1);
		GET_WIDGET (radiobutton_mouse [i], text);

		sprintf (text, "radiobutton_keyb_%d", i + 1);
		GET_WIDGET (radiobutton_keyb [i], text);

		sprintf (text, "radiobutton_joy_%d", i + 1);
		GET_WIDGET (radiobutton_joy [i], text);

		sprintf (text, "radiobutton_gen_%d", i + 1);
		GET_WIDGET (radiobutton_gen [i], text);
		gtk_widget_set_visible (widgets.radiobutton_gen [i], show_dev_options);

		sprintf (text, "notebook_%d", i + 1);
		GET_WIDGET (notebook [i], text);

		sprintf (text, "alignment_mouse_%d", i + 1);
		GET_WIDGET (alignment_mouse [i], text);

		sprintf (text, "alignment_keyb_%d", i + 1);
		GET_WIDGET (alignment_keyb [i], text);

		sprintf (text, "alignment_joy_%d", i + 1);
		GET_WIDGET (alignment_joy [i], text);

		sprintf (text, "alignment_gen_%d", i + 1);
		GET_WIDGET (alignment_gen [i], text);

		sprintf (text, "checkbutton_control_%d", i + 1);
		GET_WIDGET (checkbutton_control [i], text);

		sprintf (text, "checkbutton_shift_%d", i + 1);
		GET_WIDGET (checkbutton_shift [i], text);

		sprintf (text, "checkbutton_alt_%d", i + 1);
		GET_WIDGET (checkbutton_alt [i], text);

		sprintf (text, "checkbutton_auto_release_%d", i + 1);
		GET_WIDGET (checkbutton_auto_release [i], text);
	}

	GET_WIDGET (label_mouse_speed, "label_mouse_speed");
	GET_WIDGET (alignment_mouse_speed, "alignment_mouse_speed");
	GET_WIDGET (label_joy_displacement, "label_joy_displacement");
	GET_WIDGET (alignment_joy_displacement, "alignment_joy_displacement");
	GET_WIDGET (button_read, "button_read");
	GET_WIDGET (button_send, "button_send");

	GET_WIDGET (window_about, "window_about");
	GET_WIDGET (label_version, "label_version");
	GET_WIDGET (image_encore, "image_encore");
	GET_WIDGET (image_eneso, "image_eneso");

#undef GET_WIDGET

	gtk_builder_connect_signals (builder, &widgets);

	g_object_unref (G_OBJECT (builder));

	data_file = get_data_file ("/images/encore.png");
	gtk_image_set_from_file (GTK_IMAGE (widgets.image_encore), data_file);
	g_free (data_file);

	data_file = get_data_file ("/images/eneso.png");
	gtk_image_set_from_file (GTK_IMAGE (widgets.image_eneso), data_file);
	g_free (data_file);

	sprintf (text, "<span font=\"" FONT " 10\" foreground=\"#404040\">%s %s</span>", _("Version"), VERSION);
	gtk_label_set_markup (GTK_LABEL (widgets.label_version), text);

	/* Load the symbols images */
	for (i = 0; i < N_SWITCHES; i++)
	{
		sprintf (text, "images/symbols/%d.png", i + 1);
		data_file = get_data_file (text);

		gtk_image_set_from_file (GTK_IMAGE (widgets.image_symbol [i]), data_file);

		g_free (data_file);
	}

	/* Connect the radiobuttons */
	for (i = 0; i < N_SWITCHES; i++)
	{
		gtk_radio_button_set_group (GTK_RADIO_BUTTON (widgets.radiobutton_keyb [i]),
				gtk_radio_button_get_group (GTK_RADIO_BUTTON (widgets.radiobutton_mouse [i])));
		gtk_radio_button_set_group (GTK_RADIO_BUTTON (widgets.radiobutton_joy [i]),
				gtk_radio_button_get_group (GTK_RADIO_BUTTON (widgets.radiobutton_mouse [i])));
		gtk_radio_button_set_group (GTK_RADIO_BUTTON (widgets.radiobutton_gen [i]),
				gtk_radio_button_get_group (GTK_RADIO_BUTTON (widgets.radiobutton_mouse [i])));
	}

	/* 
	 * Create the comboboxes
	 */
	for (i = 0; i < N_SWITCHES; i++)
	{
		/* Mouse */
		widgets.combobox_mouse [i] = gtk_combo_box_new_text ();

		g_signal_connect (widgets.combobox_mouse [i],
				"changed", G_CALLBACK (combobox_action_changed_cb), GINT_TO_POINTER (i));

		for (j = 0; j < N_MOUSE_COMBO_ACTIONS; j++)
		{
			gtk_combo_box_append_text (GTK_COMBO_BOX (widgets.combobox_mouse [i]), 
					_(mouse_combo_actions [j].text));
		}
		gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_mouse [i]), 0);

		gtk_container_add (GTK_CONTAINER (widgets.alignment_mouse [i]),
				widgets.combobox_mouse [i]);
		gtk_widget_show (widgets.combobox_mouse [i]);

		/* Keyboard */
		widgets.combobox_keyb [i] = gtk_combo_box_new_text ();

		iter = g_sequence_get_begin_iter (keyb_layout [current_keyb_layout]);
		while (iter != g_sequence_get_end_iter (keyb_layout [current_keyb_layout]))
		{
			key = g_sequence_get (iter);

			gtk_combo_box_append_text (GTK_COMBO_BOX (widgets.combobox_keyb [i]), 
					_(key->text));

			iter = g_sequence_iter_next (iter);
		}
		gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_keyb [i]), 0);

		gtk_container_add (GTK_CONTAINER (widgets.alignment_keyb [i]),
				widgets.combobox_keyb [i]);
		gtk_widget_show (widgets.combobox_keyb [i]);

		/* Joystick */
		widgets.combobox_joy [i] = gtk_combo_box_new_text ();

		g_signal_connect (widgets.combobox_joy [i],
				"changed", G_CALLBACK (combobox_action_changed_cb), GINT_TO_POINTER (i));

		for (j = 0; j < N_JOY_COMBO_ACTIONS; j++)
		{
			gtk_combo_box_append_text (GTK_COMBO_BOX (widgets.combobox_joy [i]), 
					_(joy_combo_actions [j].text));
		}
		gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_joy [i]), 0);

		gtk_container_add (GTK_CONTAINER (widgets.alignment_joy [i]),
				widgets.combobox_joy [i]);
		gtk_widget_show (widgets.combobox_joy [i]);

		/* Generic */
		widgets.combobox_gen [i] = gtk_combo_box_new_text ();

		for (j = 0; j < N_GEN_COMBO_ACTIONS; j++)
		{
			gtk_combo_box_append_text (GTK_COMBO_BOX (widgets.combobox_gen [i]), 
					_(gen_combo_actions [j].text));
		}
		gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_gen [i]), 0);

		gtk_container_add (GTK_CONTAINER (widgets.alignment_gen [i]),
				widgets.combobox_gen [i]);
		gtk_widget_show (widgets.combobox_gen [i]);
	}

	/* Mouse speed */
	widgets.combobox_mouse_speed = gtk_combo_box_new_text ();

	for (i = 0; i < N_MOUSE_COMBO_SPEEDS; i++)
	{
		gtk_combo_box_append_text (GTK_COMBO_BOX (widgets.combobox_mouse_speed), 
				_(mouse_combo_speeds [i].text));
	}
	gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_mouse_speed), 0);

	gtk_container_add (GTK_CONTAINER (widgets.alignment_mouse_speed),
			widgets.combobox_mouse_speed);
	gtk_widget_show (widgets.combobox_mouse_speed);

	/* Joystick displacement */
	widgets.combobox_joy_displacement = gtk_combo_box_new_text ();

	for (i = 0; i < N_JOY_COMBO_DISPLACEMENTS; i++)
	{
		gtk_combo_box_append_text (GTK_COMBO_BOX (widgets.combobox_joy_displacement), 
				_(joy_combo_displacements [i].text));
	}
	gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_joy_displacement), 0);

	gtk_container_add (GTK_CONTAINER (widgets.alignment_joy_displacement),
			widgets.combobox_joy_displacement);
	gtk_widget_show (widgets.combobox_joy_displacement);


	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (widgets.checkmenuitem_dev_options),
			show_dev_options);
	update_config_widgets ();
}

	void
show_main_window (void)
{
	gtk_window_present (GTK_WINDOW (widgets.window_main));
}

	void
update_main_window (void)
{
	gint	i;

	/* Title */
	gtk_window_set_title (GTK_WINDOW (widgets.window_main),
			device_connected ? _("enCore") : _("enCore (disconnected)"));
	gtk_widget_queue_draw (widgets.window_main);

	/* Disable/enable widgets */
	for (i = 0; i < N_SWITCHES; i++)
	{
		gtk_widget_set_sensitive (widgets.radiobutton_mouse [i], device_connected);
		gtk_widget_set_sensitive (widgets.radiobutton_keyb [i], device_connected);
		gtk_widget_set_sensitive (widgets.radiobutton_joy [i], device_connected);
		gtk_widget_set_sensitive (widgets.radiobutton_gen [i], device_connected);

		gtk_widget_set_sensitive (widgets.combobox_mouse [i], device_connected);
		gtk_widget_set_sensitive (widgets.combobox_keyb [i], device_connected);
		gtk_widget_set_sensitive (widgets.combobox_joy [i], device_connected);
		gtk_widget_set_sensitive (widgets.combobox_gen [i], device_connected);

		gtk_widget_set_sensitive (widgets.checkbutton_control [i], device_connected);
		gtk_widget_set_sensitive (widgets.checkbutton_shift [i], device_connected);
		gtk_widget_set_sensitive (widgets.checkbutton_alt [i], device_connected);

		gtk_widget_set_sensitive (widgets.checkbutton_auto_release [i], device_connected);
	}

	gtk_widget_set_sensitive (widgets.label_mouse_speed, device_connected);
	gtk_widget_set_sensitive (widgets.combobox_mouse_speed, device_connected);
	gtk_widget_set_sensitive (widgets.label_joy_displacement, device_connected);
	gtk_widget_set_sensitive (widgets.combobox_joy_displacement, device_connected);

	gtk_widget_set_sensitive (widgets.button_read, device_connected);
	gtk_widget_set_sensitive (widgets.button_send, device_connected);
}

	void
update_config_widgets (void)
{
	GSequenceIter*	iter;
	Key*	key;
	gint	i, j;

	/* Switches */
	for (i = 0; i < N_SWITCHES; i++)
	{
		switch (switch_action [i].type)
		{
			case MOUSE_ACTION:
			case MOUSE_MOV_ACTION:

				/* Find the action in the list and set the appropiate combobox item */
				for (j = 0; j < N_MOUSE_COMBO_ACTIONS; j++)
				{
					if (switch_action [i].type == mouse_combo_actions [j].action.type &&
							switch_action [i].action == mouse_combo_actions [j].action.action)
					{
						gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_mouse [i]), j);
						break;
					}
				}

				/* Show the mouse page */
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widgets.radiobutton_mouse [i]), TRUE);
				gtk_notebook_set_current_page (GTK_NOTEBOOK (widgets.notebook [i]), PAGE_MOUSE);

				break;

			case KEYBOARD_ACTION:

				/* Find the key in the layout and set the appropiate combobox item */
				iter = g_sequence_get_begin_iter (keyb_layout [current_keyb_layout]);
				j = 0;

				while (iter != g_sequence_get_end_iter (keyb_layout [current_keyb_layout]))
				{
					key = g_sequence_get (iter);

					if (switch_action [i].action == key->code)
					{						
						gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_keyb [i]), j);
						break;
					}

					iter = g_sequence_iter_next (iter);
					j++;
				}

				/* Toggle the modifiers checkbuttons */
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widgets.checkbutton_control [i]),
						switch_action [i].modifiers & MODIF_LEFT_CTRL);
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widgets.checkbutton_shift [i]),
						switch_action [i].modifiers & MODIF_LEFT_SHIFT);
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widgets.checkbutton_alt [i]),
						switch_action [i].modifiers & MODIF_LEFT_ALT);

				/* Show the keyboard page */
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widgets.radiobutton_keyb [i]), TRUE);
				gtk_notebook_set_current_page (GTK_NOTEBOOK (widgets.notebook [i]), PAGE_KEYB);

				break;

			case JOYSTICK_ACTION:
			case JOYSTICK_MOV_ACTION:

				/* Find the action in the list and set the appropiate combobox item */
				for (j = 0; j < N_JOY_COMBO_ACTIONS; j++)
				{
					if (switch_action [i].type == joy_combo_actions [j].action.type &&
							switch_action [i].action == joy_combo_actions [j].action.action)
					{
						gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_joy [i]), j);
						break;
					}
				}

				/* Show the joystick page */
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widgets.radiobutton_joy [i]), TRUE);
				gtk_notebook_set_current_page (GTK_NOTEBOOK (widgets.notebook [i]), PAGE_JOY);

				break;


			case GENERIC_ACTION:

				/* Find the action in the list and set the appropiate combobox item */
				for (j = 0; j < N_GEN_COMBO_ACTIONS; j++)
				{
					if (switch_action [i].type == gen_combo_actions [j].action.type &&
							switch_action [i].action == gen_combo_actions [j].action.action)
					{
						gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_gen [i]), j);
						break;
					}
				}

				/* Show the generic page */
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widgets.radiobutton_gen [i]), TRUE);
				gtk_notebook_set_current_page (GTK_NOTEBOOK (widgets.notebook [i]), PAGE_GEN);

				break;
		}

		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widgets.checkbutton_auto_release [i]),
				switch_action [i].auto_release);
	}

	/* Parameters */
	for (i = 0; i < N_MOUSE_COMBO_SPEEDS; i++)
	{
		if ((mouse_speed [0] == mouse_combo_speeds [i].speed [0]) &&
				(mouse_speed [1] == mouse_combo_speeds [i].speed [1]) &&
				(mouse_speed [2] == mouse_combo_speeds [i].speed [2]))
		{
			gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_mouse_speed), i);
			break;
		}
	}

	for (i = N_JOY_COMBO_DISPLACEMENTS - 1; i >= 0; i--)
	{
		if (joy_displacement >= joy_combo_displacements [i].value)
		{
			gtk_combo_box_set_active (GTK_COMBO_BOX (widgets.combobox_joy_displacement), i);
			break;
		}
	}
}

	static void
radiobutton_type_toggled_cb (GtkToggleButton* togglebutton, gint sw, gint type)
{
	if (gtk_toggle_button_get_active (togglebutton))
	{
		gtk_notebook_set_current_page (GTK_NOTEBOOK (widgets.notebook [sw]), type);
		set_checkbutton_auto_release_visible (sw);
	}
}

#define RADIOBUTTON_TYPE_TOGGLED_CB(sw,type,cb) \
	G_MODULE_EXPORT	void cb (GtkToggleButton* togglebutton, gpointer data) \
{ radiobutton_type_toggled_cb(togglebutton, sw, type); }

RADIOBUTTON_TYPE_TOGGLED_CB (0, 0, radiobutton_mouse_1_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (1, 0, radiobutton_mouse_2_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (2, 0, radiobutton_mouse_3_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (3, 0, radiobutton_mouse_4_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (4, 0, radiobutton_mouse_5_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (5, 0, radiobutton_mouse_6_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (6, 0, radiobutton_mouse_7_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (7, 0, radiobutton_mouse_8_toggled_cb);

RADIOBUTTON_TYPE_TOGGLED_CB (0, 1, radiobutton_gen_1_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (1, 1, radiobutton_gen_2_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (2, 1, radiobutton_gen_3_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (3, 1, radiobutton_gen_4_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (4, 1, radiobutton_gen_5_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (5, 1, radiobutton_gen_6_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (6, 1, radiobutton_gen_7_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (7, 1, radiobutton_gen_8_toggled_cb);

RADIOBUTTON_TYPE_TOGGLED_CB (0, 2, radiobutton_keyb_1_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (1, 2, radiobutton_keyb_2_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (2, 2, radiobutton_keyb_3_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (3, 2, radiobutton_keyb_4_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (4, 2, radiobutton_keyb_5_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (5, 2, radiobutton_keyb_6_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (6, 2, radiobutton_keyb_7_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (7, 2, radiobutton_keyb_8_toggled_cb);

RADIOBUTTON_TYPE_TOGGLED_CB (0, 3, radiobutton_joy_1_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (1, 3, radiobutton_joy_2_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (2, 3, radiobutton_joy_3_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (3, 3, radiobutton_joy_4_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (4, 3, radiobutton_joy_5_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (5, 3, radiobutton_joy_6_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (6, 3, radiobutton_joy_7_toggled_cb);
RADIOBUTTON_TYPE_TOGGLED_CB (7, 3, radiobutton_joy_8_toggled_cb);

#undef RADIOBUTTON_TYPE_TOGGLED_CB

	static void
radiomenuitem_lang_activate_cb (GtkMenuItem* menuitem, gint language)
{
	if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem)))
	{
		if (language != current_language)
		{
			set_env_var ("LANG", language_name [language]);
			current_language = language;

			update_config_file ();

			dialog_info (_("Language"),
					_("You need to restart the application to apply this change"));
		}
	}
}

#define RADIOMENUITEM_LANG_ACTIVATE_CB(language,cb) \
	G_MODULE_EXPORT	void cb (GtkMenuItem* menuitem, gpointer data) \
{ radiomenuitem_lang_activate_cb(menuitem, language); }

RADIOMENUITEM_LANG_ACTIVATE_CB (LANG_EN, radiomenuitem_lang_en_activate_cb);
RADIOMENUITEM_LANG_ACTIVATE_CB (LANG_ES, radiomenuitem_lang_es_activate_cb);
RADIOMENUITEM_LANG_ACTIVATE_CB (LANG_IT, radiomenuitem_lang_it_activate_cb);
RADIOMENUITEM_LANG_ACTIVATE_CB (LANG_EL, radiomenuitem_lang_el_activate_cb);

#undef MENUITEM_LANG_ACTIVATE_CB

	static void
radiomenuitem_keyb_layout_activate_cb (GtkMenuItem* menuitem, gint keyb_layout)
{
	if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem)))
	{
		if (keyb_layout != current_keyb_layout)
		{
			current_keyb_layout = keyb_layout;

			update_config_file ();

			dialog_info (_("Keyboard layout"),
					_("You need to restart the application to apply this change"));
		}
	}
}

#define RADIOMENUITEM_KEYB_LAYOUT_ACTIVATE_CB(keyb_layout,cb) \
	G_MODULE_EXPORT	void cb (GtkMenuItem* menuitem, gpointer data) \
{ radiomenuitem_keyb_layout_activate_cb(menuitem, keyb_layout); }

RADIOMENUITEM_KEYB_LAYOUT_ACTIVATE_CB (KEYB_LAYOUT_EN_US, radiomenuitem_keyb_layout_en_US_activate_cb);
RADIOMENUITEM_KEYB_LAYOUT_ACTIVATE_CB (KEYB_LAYOUT_EN_UK, radiomenuitem_keyb_layout_en_UK_activate_cb);
RADIOMENUITEM_KEYB_LAYOUT_ACTIVATE_CB (KEYB_LAYOUT_EL, radiomenuitem_keyb_layout_el_activate_cb);
RADIOMENUITEM_KEYB_LAYOUT_ACTIVATE_CB (KEYB_LAYOUT_ES, radiomenuitem_keyb_layout_es_activate_cb);
RADIOMENUITEM_KEYB_LAYOUT_ACTIVATE_CB (KEYB_LAYOUT_DE, radiomenuitem_keyb_layout_de_activate_cb);
RADIOMENUITEM_KEYB_LAYOUT_ACTIVATE_CB (KEYB_LAYOUT_HE, radiomenuitem_keyb_layout_he_activate_cb);
RADIOMENUITEM_KEYB_LAYOUT_ACTIVATE_CB (KEYB_LAYOUT_IT, radiomenuitem_keyb_layout_it_activate_cb);
RADIOMENUITEM_KEYB_LAYOUT_ACTIVATE_CB (KEYB_LAYOUT_PT, radiomenuitem_keyb_layout_pt_activate_cb);

#undef MENUITEM_KEYB_LAYOUT_ACTIVATE_CB

	G_MODULE_EXPORT void
checkmenuitem_dev_options_toggled_cb (GtkCheckMenuItem* checkmenuitem, gpointer data)
{
	gint	i;

	show_dev_options = gtk_check_menu_item_get_active (checkmenuitem);
	update_config_file ();

	for (i = 0; i < N_SWITCHES; i++)
		gtk_widget_set_visible (widgets.radiobutton_gen [i], show_dev_options);
}

	G_MODULE_EXPORT void
menuitem_quick_guide_activate_cb (GtkMenuItem* menuitem, gpointer data)
{
	show_help ();
}

	G_MODULE_EXPORT void
menuitem_about_activate_cb (GtkMenuItem* menuitem, gpointer data)
{
	gtk_window_present (GTK_WINDOW (widgets.window_about));
}

	G_MODULE_EXPORT void
button_read_clicked_cb (GtkButton* button, gpointer data)
{
	gint	i;

	for (i = 0; i < N_SWITCHES; i++)
		send_command (COMMAND_SEND, SWITCH, i, 0, 0, 0, 0);

	send_command (COMMAND_SEND, PARAM, 0, 0, 0, 0, 0);
}

	G_MODULE_EXPORT void
button_send_clicked_cb (GtkButton* button, gpointer data)
{
	GSequenceIter*	iter;
	Key*	key;
	gint	i, j, k;
	gint	page;
	guchar	modif;

	/* Switch actions */
	for (i = 0; i < N_SWITCHES; i++)
	{
		page = gtk_notebook_get_current_page (GTK_NOTEBOOK (widgets.notebook [i]));

		switch (page)
		{
			case PAGE_MOUSE:

				/* Mouse */
				j = gtk_combo_box_get_active (GTK_COMBO_BOX (widgets.combobox_mouse [i]));

				send_command (COMMAND_CONFIG_SWITCH, i,
						mouse_combo_actions [j].action.type,
						mouse_combo_actions [j].action.action, 0,
						gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (
								widgets.checkbutton_auto_release [i])),
						0);

				break;

			case PAGE_KEYB:

				/* Keyboard */
				j = gtk_combo_box_get_active (GTK_COMBO_BOX (widgets.combobox_keyb [i]));

				iter = g_sequence_get_iter_at_pos (keyb_layout [current_keyb_layout], j);
				key = g_sequence_get (iter);

				modif = 0;
				if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widgets.checkbutton_control [i])))
					modif |= MODIF_LEFT_CTRL;
				if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widgets.checkbutton_shift [i])))
					modif |= MODIF_LEFT_SHIFT;
				if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widgets.checkbutton_alt [i])))
					modif |= MODIF_LEFT_ALT;

				send_command (COMMAND_CONFIG_SWITCH, i,
						KEYBOARD_ACTION, key->code, modif,
						gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (
								widgets.checkbutton_auto_release [i])),
						0);
				break;

			case PAGE_JOY:

				/* Joystick */
				j = gtk_combo_box_get_active (GTK_COMBO_BOX (widgets.combobox_joy [i]));

				send_command (COMMAND_CONFIG_SWITCH, i,
						joy_combo_actions [j].action.type,
						joy_combo_actions [j].action.action, 0,
						gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (
								widgets.checkbutton_auto_release [i])),
						0);

				break;		

			case PAGE_GEN:

				/* Generic */
				j = gtk_combo_box_get_active (GTK_COMBO_BOX (widgets.combobox_gen [i]));

				send_command (COMMAND_CONFIG_SWITCH, i,
						gen_combo_actions [j].action.type,
						gen_combo_actions [j].action.action,
						0, FALSE, 0);

				break;		
		}
	}

	/* Parameters */		
	j = gtk_combo_box_get_active (GTK_COMBO_BOX (widgets.combobox_mouse_speed));
	k = gtk_combo_box_get_active (GTK_COMBO_BOX (widgets.combobox_joy_displacement));

	send_command (COMMAND_CONFIG_PARAM,
			mouse_combo_speeds [j].speed [0],
			mouse_combo_speeds [j].speed [1],
			mouse_combo_speeds [j].speed [2],
			mouse_combo_speeds [j].stretch [0],
			mouse_combo_speeds [j].stretch [1],
			joy_combo_displacements [k].value);

	send_command (COMMAND_SAVE, 0, 0, 0, 0, 0, 0);

	dialog_info ("enCore", _("Configuration successfully written into the device."));
}

	G_MODULE_EXPORT void
menuitem_quit_activate_cb (GtkMenuItem* menuitem, gpointer data)
{
	gtk_main_quit ();
}

	static void
combobox_action_changed_cb (GtkComboBox* widget, gpointer user_data)
{
	set_checkbutton_auto_release_visible (GPOINTER_TO_INT (user_data));
}

	static void
set_checkbutton_auto_release_visible (gint sw)
{
	gint 		i, type, action;
	gboolean	visible;

	visible = FALSE;

	switch (gtk_notebook_get_current_page (GTK_NOTEBOOK (widgets.notebook [sw])))
	{
		case PAGE_MOUSE:
			i = gtk_combo_box_get_active (GTK_COMBO_BOX (widgets.combobox_mouse [sw]));
			type = mouse_combo_actions [i].action.type;
			action = mouse_combo_actions [i].action.action;

			if (type == MOUSE_ACTION &&
					(action == MOUSE_LEFT_CLICK || action == MOUSE_RIGHT_CLICK))
				visible = TRUE;
			break;

		case PAGE_KEYB:
			visible = TRUE;
			break;

		case PAGE_JOY:
			i = gtk_combo_box_get_active (GTK_COMBO_BOX (widgets.combobox_joy [sw]));
			type = joy_combo_actions [i].action.type;

			if (type == JOYSTICK_ACTION)
				visible = TRUE;
			break;		
	}

	gtk_widget_set_visible (widgets.checkbutton_auto_release [sw], visible);
}
